@@ -15,25 +15,25 @@ from datetime import datetime
 from zipfile import ZipFile, ZIP_DEFLATED
 from struct import unpack
 
-from .compatibility import str_types, force_unicode
+from .compatibility import int_types, num_types, str_types, force_unicode
 
 # Package imports.
 from . import xmlwriter
-from xlsxwriter.worksheet import Worksheet
-from xlsxwriter.chartsheet import Chartsheet
-from xlsxwriter.sharedstrings import SharedStringTable
-from xlsxwriter.format import Format
-from xlsxwriter.packager import Packager
+from .worksheet import Worksheet
+from .chartsheet import Chartsheet
+from .sharedstrings import SharedStringTable
+from .format import Format
+from .packager import Packager
 from .utility import xl_cell_to_rowcol
-from xlsxwriter.chart_area import ChartArea
-from xlsxwriter.chart_bar import ChartBar
-from xlsxwriter.chart_column import ChartColumn
-from xlsxwriter.chart_doughnut import ChartDoughnut
-from xlsxwriter.chart_line import ChartLine
-from xlsxwriter.chart_pie import ChartPie
-from xlsxwriter.chart_radar import ChartRadar
-from xlsxwriter.chart_scatter import ChartScatter
-from xlsxwriter.chart_stock import ChartStock
+from .chart_area import ChartArea
+from .chart_bar import ChartBar
+from .chart_column import ChartColumn
+from .chart_doughnut import ChartDoughnut
+from .chart_line import ChartLine
+from .chart_pie import ChartPie
+from .chart_radar import ChartRadar
+from .chart_scatter import ChartScatter
+from .chart_stock import ChartStock
 
 
 class Workbook(xmlwriter.XMLwriter):
@@ -69,6 +69,7 @@ class Workbook(xmlwriter.XMLwriter):
         self.optimization = options.get('constant_memory', False)
         self.in_memory = options.get('in_memory', False)
         self.excel2003_style = options.get('excel2003_style', False)
+        self.remove_timezone = options.get('remove_timezone', False)
         self.default_format_properties = \
             options.get('default_format_properties', {})
 
@@ -84,7 +85,7 @@ class Workbook(xmlwriter.XMLwriter):
         self.worksheets_objs = []
         self.charts = []
         self.drawings = []
-        self.sheetnames = []
+        self.sheetnames = {}
         self.formats = []
         self.xf_formats = []
         self.xf_format_indices = {}
@@ -97,7 +98,8 @@ class Workbook(xmlwriter.XMLwriter):
         self.named_ranges = []
         self.custom_colors = []
         self.doc_properties = {}
-        self.localtime = datetime.now()
+        self.custom_properties = []
+        self.createtime = datetime.utcnow()
         self.num_vml_files = 0
         self.num_comment_files = 0
         self.x_window = 240
@@ -295,6 +297,29 @@ class Workbook(xmlwriter.XMLwriter):
             self.fileclosed = 1
             self._store_workbook()
 
+    def set_size(self, width, height):
+        """
+        Set the size of a workbook window.
+
+        Args:
+            width:  Width  of the window in pixels.
+            height: Height of the window in pixels.
+
+        Returns:
+            Nothing.
+
+        """
+        # Convert the width/height to twips at 96 dpi.
+        if width:
+            self.window_width = int(width * 1440 / 96)
+        else:
+            self.window_width = 16095
+
+        if height:
+            self.window_height = int(height * 1440 / 96)
+        else:
+            self.window_height = 9660
+
     def set_properties(self, properties):
         """
         Set the document properties such as Title, Author etc.
@@ -308,6 +333,52 @@ class Workbook(xmlwriter.XMLwriter):
         """
         self.doc_properties = properties
 
+    def set_custom_property(self, name, value, property_type=None):
+        """
+        Set a custom document property.
+
+        Args:
+            name:          The name of the custom property.
+            value:         The value of the custom property.
+            property_type: The type of the custom property. Optional.
+
+        Returns:
+            Nothing.
+
+        """
+        if name is None or value is None:
+            warn("The name and value parameters must be non-None in "
+                 "set_custom_property()")
+            return -1
+
+        if property_type is None:
+            # Determine the property type from the Python type.
+            if isinstance(value, bool):
+                property_type = 'bool'
+            elif isinstance(value, datetime):
+                property_type = 'date'
+            elif isinstance(value, int_types):
+                property_type = 'number_int'
+            elif isinstance(value, num_types):
+                property_type = 'number'
+            else:
+                property_type = 'text'
+
+        if property_type == 'date':
+            value = value.strftime("%Y-%m-%dT%H:%M:%SZ")
+
+        if property_type == 'text' and len(value) > 255:
+            warn("Length of 'value' parameter exceeds Excel's limit of 255 "
+                 "characters in set_custom_property(): '%s'" %
+                 force_unicode(value))
+
+        if len(name) > 255:
+            warn("Length of 'name' parameter exceeds Excel's limit of 255 "
+                 "characters in set_custom_property(): '%s'" %
+                 force_unicode(name))
+
+        self.custom_properties.append((name, value, property_type))
+
     def set_calc_mode(self, mode, calc_id=None):
         """
         Set the Excel calculation mode for the workbook.
@@ -407,6 +478,19 @@ class Workbook(xmlwriter.XMLwriter):
         """
         return self.worksheets_objs
 
+    def get_worksheet_by_name(self, name):
+        """
+        Return a worksheet object in the workbook using the sheetname.
+
+        Args:
+            name: The name of the worksheet.
+
+        Returns:
+            A worksheet object or None.
+
+        """
+        return self.sheetnames.get(name)
+
     def use_zip64(self):
         """
         Allow ZIP64 extensions when writing xlsx file zip container.
@@ -549,7 +633,7 @@ class Workbook(xmlwriter.XMLwriter):
         sheet_index = len(self.worksheets_objs)
         name = self._check_sheetname(name, is_chartsheet)
 
-        # Initialisation data to pass to the worksheet.
+        # Initialization data to pass to the worksheet.
         init_data = {
             'name': name,
             'index': sheet_index,
@@ -565,6 +649,7 @@ class Workbook(xmlwriter.XMLwriter):
             'default_date_format': self.default_date_format,
             'default_url_format': self.default_url_format,
             'excel2003_style': self.excel2003_style,
+            'remove_timezone': self.remove_timezone,
         }
 
         if is_chartsheet:
@@ -575,7 +660,7 @@ class Workbook(xmlwriter.XMLwriter):
         worksheet._initialize(init_data)
 
         self.worksheets_objs.append(worksheet)
-        self.sheetnames.append(name)
+        self.sheetnames[name] = worksheet
 
         return worksheet
 
@@ -1047,6 +1132,12 @@ class Workbook(xmlwriter.XMLwriter):
         if not image_data:
             fh.close()
 
+        # Set a default dpi for images with 0 dpi.
+        if x_dpi == 0:
+            x_dpi = 96
+        if y_dpi == 0:
+            y_dpi = 96
+
         return image_type, width, height, image_name, x_dpi, y_dpi
 
     def _process_png(self, data):
@@ -1185,7 +1276,7 @@ class Workbook(xmlwriter.XMLwriter):
         sheetname = sheetname.strip("'")
 
         if sheetname in self.sheetnames:
-            return self.sheetnames.index(sheetname)
+            return self.sheetnames[sheetname].index
         else:
             return None
 
